from .notification import Notification, common_notification_paramenters


class FilterNotification(Notification):
    """Notification based on specific filters

    Attributes:
        filters
        {common_notification_paramenters}
    """.format(common_notification_paramenters=common_notification_paramenters)

    def __init__(self, filters, **kwargs):
        Notification.__init__(self, **kwargs)
        self.filters = []

        for filter in filters:
            if filter == "OR":
                self.filters.append({"operator": "OR"})

            elif filter == "AND":
                continue

            else:
                self.filters.append(filter.data)

    def get_data(self):
        return {
            **self.get_common_data(),
            "filters": self.filters
        }



class Filter:
    pass


class LastSession(Filter):
    """Filtered by the last session of the user

    Arguments:
        relation: "<" or ">"
        hours_ago: number of hours before or after the users last session
    """

    def __init__(self, relation, hours_ago):
        self.data = {
            "field": "last_session",
            "relation": relation,
            "hours_ago": hours_ago
        }


class FirstSession(Filter):
    """Filtered by the first session of the user

    Arguments:
        relation: "<" or ">"
        hours_ago: number of hours before or after the users last session
    """

    def __init__(self, relation, hours_ago):
        self.data = {
            "field": "first_session",
            "relation": relation,
            "hours_ago": hours_ago
        }


class SessionCount(Filter):
    """Filtered by amount of sessions

    Arguments:
        relation: "<", ">", "=" or "!="
        value: time in seconds the user has been in your app
    """

    def __init__(self, relation, value):
        self.data = {
            "field": "session_count",
            "value": value
        }


class SessionTime(Filter):
    """Filtered by time the user has been in app

    Arguments:
        relation: "<" or ">"
        value: time in seconds the user has been in your app
    """

    def __init__(self, relation, value):
        self.data = {
            "field": "session_time",
            "value": value
        }


class AmountSpent(Filter):
    """Filtered by amount spent on IAP

    Arguments:
        relation: "<", ">" or "="
        value: amount in USD a user has spent on IAP (In App Purchases)
    """

    def __init__(self, relation, value):
        self.data = {
            "field": "amount_spent",
            "value": value
        }


class BoughtSku(Filter):
    """Filtered by SKU purchased

    Arguments:
        relation: "<",  ">" or "="
        key: SKU purchased in your app as an IAP (In App Purchases)
        value: value of SKU to compare to
    """

    def __init__(self, relation, key, value):
        self.data = {
            "field": "bought_sku",
            "relation": relation,
            "key": key,
            "value": value
        }


class Tag(Filter):
    """Filtered by tag

    Arguments:
        relation: "<", ">", "=", "!=", "exists" or "not_exists"
        key: tag to compare
        value: time in seconds the user has been in your app
    """

    def __init__(self, key, relation, value):
        self.data = {
            "field": "tag",
            "key": key,
            "relation": relation,
            "value": value
        }


class Language(Filter):
    """Filtered by language of user

    Arguments:
        relation: "=" or "!="
        value: 2 character language code
    """

    def __init__(self, relation, value):
        self.data = {
            "field": "language",
            "relation": relation,
            "value": value
        }


class AppVersion(Filter):
    """Filtered by version of the app

    Arguments:
        relation: "<", ">", "=" or "!="
        value: app version
    """

    def __init__(self, relation, value):
        self.data = {
            "field": "app_version",
            "relation": relation,
            "value": value
        }


class Location(Filter):
    """Filtered by location of the user

    Arguments:
        radius: in meters
        lat: latitude
        long: longitude
    """

    def __init__(self, radius, lat, long):
        self.data = {
            "field": "location",
            "radius": radius,
            "lat": lat,
            "long": long
        }


class Email(Filter):
    """Filtered by time the user has been in app

    For email targeting only, not used to send push notifications

    Arguments:
        value: email address
    """

    def __init__(self, value):
        self.data = {
            "field": "email",
            "value": value
        }


class Country(Filter):
    """Filtered by country of the user

    Arguments:
        relation: "="
        value: 2 character country code
    """

    def __init__(self, value):
        self.data = {
            "field": "country",
            "value": value
        }



from .notification import Notification, common_notification_paramenters


class DeviceNotification(Notification):
    """Notification to a specific device using their id

    Attributes:
        include_player_ids
        include_email_tokens
        include_ios_tokens
        include_wp_urls
        include_wp_wns_uris
        include_amazon_reg_ids
        include_chrome_reg_ids
        include_chrome_web_reg_ids
        include_android_reg_ids
        include_external_user_ids
        {common_notification_paramenters}
    """.format(common_notification_paramenters=common_notification_paramenters)

    def __init__(self,
                 include_player_ids=None,
                 include_email_tokens=None,
                 include_ios_tokens=None,
                 include_wp_urls=None,
                 include_wp_wns_uris=None,
                 include_amazon_reg_ids=None,
                 include_chrome_reg_ids=None,
                 include_chrome_web_reg_ids=None,
                 include_android_reg_ids=None,
                 include_external_user_ids=None,
                 **kwargs):
        Notification.__init__(self, **kwargs)
        self.include_player_ids = include_player_ids
        self.include_email_tokens = include_email_tokens
        self.include_ios_tokens = include_ios_tokens
        self.include_wp_urls = include_wp_urls
        self.include_wp_wns_uris = include_wp_wns_uris
        self.include_amazon_reg_ids = include_amazon_reg_ids
        self.include_chrome_reg_ids = include_chrome_reg_ids
        self.include_chrome_web_reg_ids = include_chrome_web_reg_ids
        self.include_android_reg_ids = include_android_reg_ids
        self.include_external_user_ids = include_external_user_ids

    def get_data(self):
        return {
            **self.get_common_data(),
            "include_player_ids": self.include_player_ids,
            "include_email_tokens": self.include_email_tokens,
            "include_ios_tokens": self.include_ios_tokens,
            "include_wp_urls": self.include_wp_urls,
            "include_wp_wns_uris": self.include_wp_wns_uris,
            "include_amazon_reg_ids": self.include_amazon_reg_ids,
            "include_chrome_reg_ids": self.include_chrome_reg_ids,
            "include_chrome_web_reg_ids": self.include_chrome_web_reg_ids,
            "include_android_reg_ids": self.include_android_reg_ids,
            "include_external_user_ids": self.include_external_user_ids
        }



import requests

from .errors import OneSignalAPIError


class OneSignal:
    """Connects all the functions and methods to the OneSignal API

    Central class that is used to interact with the OneSignal API
    by passing notification classes into the methods.

    Attributes:
        app_id: OneSignal app id
        rest_api_key: secret OneSignal rest api key
    """
    base_api = "https://onesignal.com/api/v1/"

    def __init__(self, app_id, rest_api_key):
        """Inits OneSignal with connection details"""
        self.app_id = app_id
        self.rest_api_key = rest_api_key

    def request(self, method, endpoint, json={}):
        """Sends a request to the OneSignal API

        Args:
            method: HTTP request method
            endpoint: api endpoint
            json: request data

        Returns:
            A dict of the api response

        Raises:
            OneSignalAPIError: OneSignal API request was not successful
        """

        r = requests.request(
            method,
            self.base_api + endpoint,
            json=json,
            headers={
                "Authorization": "Basic " + self.rest_api_key
            }
        )

        if r.status_code != 200:
            raise OneSignalAPIError(r.json())

        return r.json()

    def send(self, notification):
        """Send a notification

        Attributes:
            notification: instance of a *Notification class

        Returns:
            A dict of the API response
        """
        # Allow unicode app ID, Python 3 doesn't support basestring
        try:
            basestring
        except NameError:
            basestring = str

        if isinstance(self.app_id, basestring):
            app_id_obj = {"app_id": self.app_id}
        elif isinstance(self.app_id, list):
            app_id_obj = {"app_ids": self.app_id}

        data = {
            **notification.get_data(),
            **app_id_obj
        }

        response = self.request("post", "notifications", json=data)

        notification.id = response["id"]

        return response

    def cancel(self, notification):
        """Cancel a notification

        Attributes:
            notification: instance of a *Notification class or notification id

        Returns:
            A dict of the API response
        """

        if isinstance(notification, str):
            notification_id = notification
        else:
            if not notification.id:
                raise ValueError("The notification was propably not sent yet")
            notification_id = notification.id

        response = self.request(
            "delete",
            "notifications/" + notification_id + "?app_id=" + self.app_id
        )

        return response

    def details(self, notification):
        """Get details about a notification

        Attributes:
            notification: instance of a *Notification class or notification id

        Returns:
            A dict of the API response
        """

        if isinstance(notification, str):
            notification_id = notification
        else:
            if not notification.id:
                raise ValueError("The notification was propably not sent yet")
            notification_id = notification.id

        response = self.request(
            "get",
            "notifications/" + notification_id + "?app_id=" + self.app_id
        )

        result = {}
        for key in response.keys():
            result[self.to_underscore(key)] = response[key]

        return result

    def to_underscore(self, var):
        """Converts camelCase to underscore

        Attributes:
            var: name of variable in camelCase
        """

        result = ""
        for letter in var:
            if letter == letter.lower():
                result += letter
            else:
                result += "_" + letter.lower()
        return result



from .notification import Notification, common_notification_paramenters
from itertools import chain


class SegmentNotification(Notification):
    """Notification based on specific filters

    Attributes:
        included_segment
        excluded_segments
        {common_notification_paramenters}
    """.format(common_notification_paramenters=common_notification_paramenters)

    ALL = "All"
    ACTIVE_USERS = "Active Users"
    ENGAGED_USERS = "Engaged Users"
    INACTIVE_USERS = "Inactive Users"

    def __init__(self,
                 included_segments=None,
                 excluded_segments=None,
                 **kwargs):
        Notification.__init__(self, **kwargs)
        self.included_segments = included_segments
        self.excluded_segments = excluded_segments

    def get_data(self):
        return dict(chain({
            "included_segments": self.included_segments,
            "excluded_segments": self.excluded_segments
        }.items(), self.get_common_data().items()))



def merge_dicts(first_dict, *dicts):
    """
    Merge several dicts

    This function is needed for Python 2 support. It will be REMOVED on January 1, 2020.
    Python 3 alternative: x = {**a, **b}
    """
    new_dict = first_dict.copy()
    for dict_ in dicts:
        new_dict.update(dict_)
    return new_dict
